@@ -17,6 +17,7 @@ public:
 	void readSettings(device_data_t *data);
 
 	enum states {
+		OPEN,
 		INITIAL,
 		READING,
 		WRITING,
@@ -30,13 +31,17 @@ public:
 
 	QString lastError;
 	states currentState;
-	device_data_t *m_data;
 	void saveDeviceDetails(DeviceDetails *details, device_data_t *data);
 	void fetchDeviceDetails();
 	bool saveXMLBackup(QString fileName, DeviceDetails *details, device_data_t *data);
 	bool restoreXMLBackup(QString fileName, DeviceDetails *details);
 	void startFirmwareUpdate(QString fileName, device_data_t *data);
 	void resetSettings(device_data_t *data);
+
+	QString dc_open(device_data_t *data);
+public
+slots:
+	void dc_close(device_data_t *data);
 signals:
 	void progress(int percent);
 	void message(QString msg);
