@@ -900,7 +900,8 @@ void update_setpoint_events(struct divecomputer *dc)
 
 	if (dc->divemode == OC &&
 	    (same_string(dc->model, "Shearwater Predator") ||
-	     same_string(dc->model, "Shearwater Petrel"))) {
+	     same_string(dc->model, "Shearwater Petrel") ||
+	     same_string(dc->model, "Shearwater Nerd"))) {
 		// make sure there's no setpoint in the samples
 		// this is an irreversible change - so switching a dive to OC
 		// by mistake when it's actually CCR is _bad_
@@ -1242,6 +1243,23 @@ static void fixup_dc_events(struct divecomputer *dc)
 	}
 }
 
+static int interpolate_depth(struct divecomputer *dc, int idx, int lastdepth, int lasttime, int now)
+{
+	int i;
+	int nextdepth = lastdepth;
+	int nexttime = now;
+
+	for (i = idx+1; i < dc->samples; i++) {
+		struct sample *sample = dc->sample + i;
+		if (sample->depth.mm < 0)
+			continue;
+		nextdepth = sample->depth.mm;
+		nexttime = sample->time.seconds;
+		break;
+	}
+	return interpolate(lastdepth, nextdepth, now-lasttime, nexttime-lasttime);
+}
+
 static void fixup_dive_dc(struct dive *dive, struct divecomputer *dc)
 {
 	int i, j;
@@ -1275,6 +1293,11 @@ static void fixup_dive_dc(struct dive *dive, struct divecomputer *dc)
 		int o2_pressure = sample->o2cylinderpressure.mbar;
 		int index;
 
+		if (depth < 0) {
+			depth = interpolate_depth(dc, i, lastdepth, lasttime, time);
+			sample->depth.mm = depth;
+		}
+
 		/* if we have an explicit first cylinder */
 		if (sample->sensor == 0 && first_cylinder != 0)
 			sample->sensor = first_cylinder;
