@@ -10,6 +10,8 @@
 
 #include "dive.h"
 #include "file.h"
+#include "git-access.h"
+#include "qthelperfromc.h"
 
 /* For SAMPLE_* */
 #include <libdivecomputer/parser.h>
@@ -83,7 +85,7 @@ static void zip_read(struct zip_file *file, const char *filename)
 	free(mem);
 }
 
-static int try_to_open_zip(const char *filename, struct memblock *mem)
+int try_to_open_zip(const char *filename, struct memblock *mem)
 {
 	int success = 0;
 	/* Grr. libzip needs to re-open the file, it can't take a buffer */
@@ -169,12 +171,13 @@ static int try_to_open_db(const char *filename, struct memblock *mem)
 	char dm5_test[] = "select count(*) from sqlite_master where type='table' and name='Dive' and sql like '%SampleBlob%'";
 	char shearwater_test[] = "select count(*) from sqlite_master where type='table' and name='system' and sql like '%dbVersion%'";
 	char cobalt_test[] = "select count(*) from sqlite_master where type='table' and name='TrackPoints' and sql like '%DepthPressure%'";
+	char divinglog_test[] = "select count(*) from sqlite_master where type='table' and name='DBInfo' and sql like '%PrgName%'";
 	int retval;
 
 	retval = sqlite3_open(filename, &handle);
 
 	if (retval) {
-		fprintf(stderr, translate("gettextFromC", "Database connection failed '%s'.\n"), filename);
+		fprintf(stderr, "Database connection failed '%s'.\n", filename);
 		return 1;
 	}
 
@@ -210,6 +213,14 @@ static int try_to_open_db(const char *filename, struct memblock *mem)
 		return retval;
 	}
 
+	/* Testing if DB schema resembles Divinglog database format */
+	retval = sqlite3_exec(handle, divinglog_test, &db_test_func, 0, NULL);
+	if (!retval) {
+		retval = parse_divinglog_buffer(handle, filename, mem->buffer, mem->size, &dive_table);
+		sqlite3_close(handle);
+		return retval;
+	}
+
 	sqlite3_close(handle);
 
 	return retval;
@@ -425,20 +436,30 @@ int parse_file(const char *filename)
 	char *fmt;
 	int ret;
 
-	git = is_git_repository(filename, &branch);
+	git = is_git_repository(filename, &branch, NULL, false);
+	if (prefs.cloud_git_url &&
+	    strstr(filename, prefs.cloud_git_url)
+	    && git == dummy_git_repository)
+		/* opening the cloud storage repository failed for some reason
+		 * give up here and don't send errors about git repositories */
+		return 0;
+
 	if (git && !git_load_dives(git, branch))
 		return 0;
 
-	if (readfile(filename, &mem) < 0) {
-		/* we don't want to display an error if this was the default file */
-		if (prefs.default_filename && !strcmp(filename, prefs.default_filename))
+	if ((ret = readfile(filename, &mem)) < 0) {
+		/* we don't want to display an error if this was the default file or the cloud storage */
+		if ((prefs.default_filename && !strcmp(filename, prefs.default_filename)) ||
+		    isCloudUrl(filename))
 			return 0;
 
 		return report_error(translate("gettextFromC", "Failed to read '%s'"), filename);
+	} else if (ret == 0) {
+		return report_error(translate("gettextFromC", "Empty file '%s'"), filename);
 	}
 
 	fmt = strrchr(filename, '.');
-	if (fmt && (!strcasecmp(fmt + 1, "DB") || !strcasecmp(fmt + 1, "BAK"))) {
+	if (fmt && (!strcasecmp(fmt + 1, "DB") || !strcasecmp(fmt + 1, "BAK") || !strcasecmp(fmt + 1, "SQL"))) {
 		if (!try_to_open_db(filename, &mem)) {
 			free(mem.buffer);
 			return 0;
@@ -454,6 +475,18 @@ int parse_file(const char *filename)
 		return -1;
 	}
 
+	/* DataTrak/Wlog */
+	if (fmt && !strcasecmp(fmt + 1, "LOG")) {
+		datatrak_import(filename, &dive_table);
+		return 0;
+	}
+
+	/* OSTCtools */
+	if (fmt && (!strcasecmp(fmt + 1, "DIVE"))) {
+		ostctools_import(filename, &dive_table);
+		return 0;
+	}
+
 	ret = parse_file_buffer(filename, &mem);
 	free(mem.buffer);
 	return ret;
@@ -477,7 +510,7 @@ char *parse_mkvi_value(const char *haystack, const char *needle)
 				terminator = '\r';
 			}
 			*endptr = 0;
-			ret = strdup(valueptr);
+			ret = copy_string(valueptr);
 			*endptr = terminator;
 
 		}
@@ -491,12 +524,11 @@ char *next_mkvi_key(const char *haystack)
 
 	if ((valueptr = strstr(haystack, "\n")) != NULL) {
 		valueptr += 1;
-	}
-	if ((endptr = strstr(valueptr, ": ")) != NULL) {
-		*endptr = 0;
-		ret = strdup(valueptr);
-		*endptr = ':';
-
+		if ((endptr = strstr(valueptr, ": ")) != NULL) {
+			*endptr = 0;
+			ret = strdup(valueptr);
+			*endptr = ':';
+		}
 	}
 	return ret;
 }
@@ -520,16 +552,18 @@ int parse_txt_file(const char *filename, const char *csv)
 		bool has_depth = false, has_setpoint = false, has_ndl = false;
 		char *lineptr, *key, *value;
 		int o2cylinder_pressure = 0, cylinder_pressure = 0, cur_cylinder_index = 0;
+		unsigned int prev_time = 0;
 
 		struct dive *dive;
 		struct divecomputer *dc;
 		struct tm cur_tm;
 
-		if (sscanf(parse_mkvi_value(memtxt.buffer, "Dive started at"), "%d-%d-%d %d:%d:%d",
-					&y, &m, &d, &hh, &mm, &ss) != 6) {
+		value = parse_mkvi_value(memtxt.buffer, "Dive started at");
+		if (sscanf(value, "%d-%d-%d %d:%d:%d", &y, &m, &d, &hh, &mm, &ss) != 6) {
+			free(value);
 			return -1;
 		}
-
+		free(value);
 		cur_tm.tm_year = y;
 		cur_tm.tm_mon = m - 1;
 		cur_tm.tm_mday = d;
@@ -600,6 +634,7 @@ int parse_txt_file(const char *filename, const char *csv)
 		 */
 
 		if (readfile(csv, &memcsv) < 0) {
+			free(dive);
 			return report_error(translate("gettextFromC", "Poseidon import failed: unable to read '%s'"), csv);
 		}
 		lineptr = memcsv.buffer;
@@ -617,7 +652,14 @@ int parse_txt_file(const char *filename, const char *csv)
 			has_setpoint = false;
 			has_ndl = false;
 			sample = prepare_sample(dc);
-			sample->time.seconds = cur_sampletime;
+
+			/*
+			 * There was a bug in MKVI download tool that resulted in erroneous sample
+			 * times. This fix should work similarly as the vendor's own.
+			 */
+
+			sample->time.seconds = cur_sampletime < 0xFFFF * 3 / 4 ? cur_sampletime : prev_time;
+			prev_time = sample->time.seconds;
 
 			do {
 				int i = sscanf(lineptr, "%d,%d,%d", &sampletime, &type, &value);
@@ -644,6 +686,7 @@ int parse_txt_file(const char *filename, const char *csv)
 									QT_TRANSLATE_NOOP("gettextFromC", "Mouth piece position not connected"));
 							break;
 						}
+						break;
 					case 3:
 						//Power Off event
 						add_event(dc, cur_sampletime, 0, 0, 0,
@@ -791,83 +834,17 @@ int parse_txt_file(const char *filename, const char *csv)
 	return 0;
 }
 
-#define MAXCOLDIGITS 3
-#define MAXCOLS 100
+#define MAXCOLDIGITS 10
 #define DATESTR 9
 #define TIMESTR 6
-void init_csv_file_parsing(char **params, char *timebuf, char *depthbuf, char *tempbuf, char *po2buf, char *cnsbuf, char *ndlbuf, char *ttsbuf, char *stopdepthbuf, char *pressurebuf, char *unitbuf, char *separator_index, time_t *now, struct tm *timep, char *curdate, char *curtime, int timef, int depthf, int tempf, int po2f, int cnsf, int ndlf, int ttsf, int stopdepthf, int pressuref, int sepidx, const char *csvtemplate, int unitidx)
+
+int parse_csv_file(const char *filename, char **params, int pnr, const char *csvtemplate)
 {
-	int pnr = 0;
-
-	snprintf(timebuf, MAXCOLDIGITS, "%d", timef);
-	snprintf(depthbuf, MAXCOLDIGITS, "%d", depthf);
-	snprintf(tempbuf, MAXCOLDIGITS, "%d", tempf);
-	snprintf(po2buf, MAXCOLDIGITS, "%d", po2f);
-	snprintf(cnsbuf, MAXCOLDIGITS, "%d", cnsf);
-	snprintf(ndlbuf, MAXCOLDIGITS, "%d", ndlf);
-	snprintf(ttsbuf, MAXCOLDIGITS, "%d", ttsf);
-	snprintf(stopdepthbuf, MAXCOLDIGITS, "%d", stopdepthf);
-	snprintf(pressurebuf, MAXCOLDIGITS, "%d", pressuref);
-	snprintf(separator_index, MAXCOLDIGITS, "%d", sepidx);
-	snprintf(unitbuf, MAXCOLDIGITS, "%d", unitidx);
-	time(now);
-	timep = localtime(now);
-	strftime(curdate, DATESTR, "%Y%m%d", timep);
-
-	/* As the parameter is numeric, we need to ensure that the leading zero
-	* is not discarded during the transform, thus prepend time with 1 */
-	strftime(curtime, TIMESTR, "1%H%M", timep);
-
-	params[pnr++] = "timeField";
-	params[pnr++] = timebuf;
-	params[pnr++] = "depthField";
-	params[pnr++] = depthbuf;
-	params[pnr++] = "tempField";
-	params[pnr++] = tempbuf;
-	params[pnr++] = "po2Field";
-	params[pnr++] = po2buf;
-	params[pnr++] = "cnsField";
-	params[pnr++] = cnsbuf;
-	params[pnr++] = "ndlField";
-	params[pnr++] = ndlbuf;
-	params[pnr++] = "ttsField";
-	params[pnr++] = ttsbuf;
-	params[pnr++] = "stopdepthField";
-	params[pnr++] = stopdepthbuf;
-	params[pnr++] = "pressureField";
-	params[pnr++] = pressurebuf;
-	params[pnr++] = "date";
-	params[pnr++] = curdate;
-	params[pnr++] = "time";
-	params[pnr++] = curtime;
-	params[pnr++] = "units";
-	params[pnr++] = unitbuf;
-	params[pnr++] = "separatorIndex";
-	params[pnr++] = separator_index;
-	params[pnr++] = NULL;
-}
-
-int parse_csv_file(const char *filename, int timef, int depthf, int tempf, int po2f, int cnsf, int ndlf, int ttsf, int stopdepthf, int pressuref, int sepidx, const char *csvtemplate, int unitidx)
-{
-	int ret;
+	int ret, i;
 	struct memblock mem;
-	char *params[27];
-	char timebuf[MAXCOLDIGITS];
-	char depthbuf[MAXCOLDIGITS];
-	char tempbuf[MAXCOLDIGITS];
-	char po2buf[MAXCOLDIGITS];
-	char cnsbuf[MAXCOLDIGITS];
-	char ndlbuf[MAXCOLDIGITS];
-	char ttsbuf[MAXCOLDIGITS];
-	char stopdepthbuf[MAXCOLDIGITS];
-	char pressurebuf[MAXCOLDIGITS];
-	char unitbuf[MAXCOLDIGITS];
-	char separator_index[MAXCOLDIGITS];
 	time_t now;
 	struct tm *timep = NULL;
-	char curdate[DATESTR];
-	char curtime[TIMESTR];
-	int previous;
+	char tmpbuf[MAXCOLDIGITS];
 
 	/* Increase the limits for recursion and variables on XSLT
 	 * parsing */
@@ -876,57 +853,81 @@ int parse_csv_file(const char *filename, int timef, int depthf, int tempf, int p
 	xsltMaxVars = 150000;
 #endif
 
-	if (timef >= MAXCOLS || depthf >= MAXCOLS || tempf >= MAXCOLS || po2f >= MAXCOLS || cnsf >= MAXCOLS || ndlf >= MAXCOLS || cnsf >= MAXCOLS || stopdepthf >= MAXCOLS || pressuref >= MAXCOLS)
-		return report_error(translate("gettextFromC", "Maximum number of supported columns on CSV import is %d"), MAXCOLS);
-
-	init_csv_file_parsing(params, timebuf, depthbuf, tempbuf, po2buf, cnsbuf,ndlbuf, ttsbuf, stopdepthbuf, pressurebuf, unitbuf, separator_index, &now, timep, curdate, curtime, timef, depthf, tempf, po2f, cnsf, ndlf, ttsf, stopdepthf, pressuref, sepidx, csvtemplate, unitidx);
-
 	if (filename == NULL)
 		return report_error("No CSV filename");
 
+	time(&now);
+	timep = localtime(&now);
+
+	strftime(tmpbuf, MAXCOLDIGITS, "%Y%m%d", timep);
+	params[pnr++] = "date";
+	params[pnr++] = strdup(tmpbuf);
+
+	/* As the parameter is numeric, we need to ensure that the leading zero
+	* is not discarded during the transform, thus prepend time with 1 */
+
+	strftime(tmpbuf, MAXCOLDIGITS, "1%H%M", timep);
+	params[pnr++] = "time";
+	params[pnr++] = strdup(tmpbuf);
+	params[pnr++] = NULL;
+
 	mem.size = 0;
 	if (try_to_xslt_open_csv(filename, &mem, csvtemplate))
 		return -1;
 
-	previous = dive_table.nr;
+	/*
+	 * Lets print command line for manual testing with xsltproc if
+	 * verbosity level is high enough. The printed line needs the
+	 * input file added as last parameter.
+	 */
+
+	if (verbose >= 2) {
+		fprintf(stderr, "(echo '<csv>'; cat %s;echo '</csv>') | xsltproc ", filename);
+		for (i=0; params[i]; i+=2)
+			fprintf(stderr, "--stringparam %s %s ", params[i], params[i+1]);
+		fprintf(stderr, "%s/xslt/csv2xml.xslt -\n", SUBSURFACE_SOURCE);
+	}
+
 	ret = parse_xml_buffer(filename, mem.buffer, mem.size, &dive_table, (const char **)params);
 
-	// mark imported dives as imported from CSV
-	for (int i = previous; i < dive_table.nr; i++)
-		if (same_string(get_dive(i)->dc.model, ""))
-			get_dive(i)->dc.model = copy_string("Imported from CSV");
-
 	free(mem.buffer);
+	for (i = 0; params[i]; i += 2)
+		free(params[i + 1]);
+
 	return ret;
 }
 
-int parse_seabear_csv_file(const char *filename, int timef, int depthf, int tempf, int po2f, int cnsf, int ndlf, int ttsf, int stopdepthf, int pressuref, int sepidx, const char *csvtemplate, int unitidx)
+#define SBPARAMS 40
+int parse_seabear_csv_file(const char *filename, char **params, int pnr, const char *csvtemplate)
 {
-	int ret;
+	int ret, i;
 	struct memblock mem;
-	char *params[27];
-	char timebuf[MAXCOLDIGITS];
-	char depthbuf[MAXCOLDIGITS];
-	char tempbuf[MAXCOLDIGITS];
-	char po2buf[MAXCOLDIGITS];
-	char cnsbuf[MAXCOLDIGITS];
-	char ndlbuf[MAXCOLDIGITS];
-	char ttsbuf[MAXCOLDIGITS];
-	char stopdepthbuf[MAXCOLDIGITS];
-	char pressurebuf[MAXCOLDIGITS];
-	char unitbuf[MAXCOLDIGITS];
-	char separator_index[MAXCOLDIGITS];
 	time_t now;
 	struct tm *timep = NULL;
-	char curdate[DATESTR];
-	char curtime[TIMESTR];
 	char *ptr, *ptr_old = NULL;
-	char *NL;
+	char *NL = NULL;
+	char tmpbuf[MAXCOLDIGITS];
 
-	if (timef >= MAXCOLS || depthf >= MAXCOLS || tempf >= MAXCOLS || po2f >= MAXCOLS || cnsf >= MAXCOLS || ndlf >= MAXCOLS || cnsf >= MAXCOLS || stopdepthf >= MAXCOLS || pressuref >= MAXCOLS)
-		return report_error(translate("gettextFromC", "Maximum number of supported columns on CSV import is %d"), MAXCOLS);
+	/* Increase the limits for recursion and variables on XSLT
+	 * parsing */
+	xsltMaxDepth = 30000;
+#if LIBXSLT_VERSION > 10126
+	xsltMaxVars = 150000;
+#endif
+
+	time(&now);
+	timep = localtime(&now);
+
+	strftime(tmpbuf, MAXCOLDIGITS, "%Y%m%d", timep);
+	params[pnr++] = "date";
+	params[pnr++] = strdup(tmpbuf);
+
+	/* As the parameter is numeric, we need to ensure that the leading zero
+	* is not discarded during the transform, thus prepend time with 1 */
+	strftime(tmpbuf, MAXCOLDIGITS, "1%H%M", timep);
+	params[pnr++] = "time";
+	params[pnr++] = strdup(tmpbuf);
 
-	init_csv_file_parsing(params, timebuf, depthbuf, tempbuf, po2buf, cnsbuf,ndlbuf, ttsbuf, stopdepthbuf, pressurebuf, unitbuf, separator_index, &now, timep, curdate, curtime, timef, depthf, tempf, po2f, cnsf, ndlf, ttsf, stopdepthf, pressuref, sepidx, csvtemplate, unitidx);
 
 	if (filename == NULL)
 		return report_error("No CSV filename");
@@ -942,46 +943,60 @@ int parse_seabear_csv_file(const char *filename, int timef, int depthf, int temp
 		NL = "\r\n";
 	}
 
-	/*
-	 * If file does not contain empty lines, it is not a valid
-	 * Seabear CSV file.
-	 */
-	if (!ptr)
-		return -1;
-
 	if (!ptr_old) {
+		ptr = mem.buffer;
 		while ((ptr = strstr(ptr, "\n\n")) != NULL) {
 			ptr_old = ptr;
 			ptr += 1;
+			NL = "\n";
 		}
 		ptr_old += 2;
-		NL = "\n";
 	} else
 		ptr_old += 4;
 
+	/*
+	 * If file does not contain empty lines, it is not a valid
+	 * Seabear CSV file.
+	 */
+	if (NL == NULL)
+		return -1;
+
 	/*
 	 * On my current sample of Seabear DC log file, the date is
 	 * without any identifier. Thus we must search for the previous
-	 * line and step through from there.
+	 * line and step through from there. That is the line after
+	 * Serial number.
 	 */
 	ptr = strstr(mem.buffer, "Serial number:");
 	if (ptr)
 		ptr = strstr(ptr, NL);
 
-	/* Write date and time values to params array */
+	/*
+	 * Write date and time values to params array, if available in
+	 * the CSV header
+	 */
+
 	if (ptr) {
 		ptr += strlen(NL) + 2;
-		memcpy(params[19], ptr, 4);
-		memcpy(params[19] + 4, ptr + 5, 2);
-		memcpy(params[19] + 6, ptr + 8, 2);
-		params[19][8] = 0;
+		/*
+		 * pnr is the index of NULL on the params as filled by
+		 * the init function. The two last entries should be
+		 * date and time. Here we overwrite them with the data
+		 * from the CSV header.
+		 */
 
-		params[21][0] = '1';
-		memcpy(params[21] + 1, ptr + 11, 2);
-		memcpy(params[21] + 3, ptr + 14, 2);
-		params[21][5] = 0;
+		memcpy(params[pnr - 3], ptr, 4);
+		memcpy(params[pnr - 3] + 4, ptr + 5, 2);
+		memcpy(params[pnr - 3] + 6, ptr + 8, 2);
+		params[pnr - 3][8] = 0;
+
+		memcpy(params[pnr - 1] + 1, ptr + 11, 2);
+		memcpy(params[pnr - 1] + 3, ptr + 14, 2);
+		params[pnr - 1][5] = 0;
 	}
 
+	params[pnr++] = NULL;
+
 	/* Move the CSV data to the start of mem buffer */
 	memmove(mem.buffer, ptr_old, mem.size - (ptr_old - (char*)mem.buffer));
 	mem.size = (int)mem.size - (ptr_old - (char*)mem.buffer);
@@ -989,73 +1004,37 @@ int parse_seabear_csv_file(const char *filename, int timef, int depthf, int temp
 	if (try_to_xslt_open_csv(filename, &mem, csvtemplate))
 		return -1;
 
+	/*
+	 * Lets print command line for manual testing with xsltproc if
+	 * verbosity level is high enough. The printed line needs the
+	 * input file added as last parameter.
+	 */
+
+	if (verbose >= 2) {
+		fprintf(stderr, "xsltproc ");
+		for (i=0; params[i]; i+=2)
+			fprintf(stderr, "--stringparam %s %s ", params[i], params[i+1]);
+		fprintf(stderr, "xslt/csv2xml.xslt\n");
+	}
+
 	ret = parse_xml_buffer(filename, mem.buffer, mem.size, &dive_table, (const char **)params);
 	free(mem.buffer);
+	for (i = 0; params[i]; i += 2)
+		free(params[i + 1]);
+
 	return ret;
 }
 
-int parse_manual_file(const char *filename, int sepidx, int units, int dateformat, int durationformat, int numberf, int datef, int timef, int durationf, int locationf, int gpsf, int maxdepthf, int meandepthf, int divemasterf, int buddyf, int notesf, int weightf, int tagsf, int cylsizef, int startpresf, int endpresf, int o2f, int hef, int airtempf, int watertempf)
+int parse_manual_file(const char *filename, char **params, int pnr)
 {
 	struct memblock mem;
-	int pnr = 0;
-	char *params[53];
-	char numberbuf[MAXCOLDIGITS];
-	char datebuf[MAXCOLDIGITS];
-	char timebuf[MAXCOLDIGITS];
-	char durationbuf[MAXCOLDIGITS];
-	char locationbuf[MAXCOLDIGITS];
-	char gpsbuf[MAXCOLDIGITS];
-	char maxdepthbuf[MAXCOLDIGITS];
-	char meandepthbuf[MAXCOLDIGITS];
-	char divemasterbuf[MAXCOLDIGITS];
-	char buddybuf[MAXCOLDIGITS];
-	char notesbuf[MAXCOLDIGITS];
-	char weightbuf[MAXCOLDIGITS];
-	char tagsbuf[MAXCOLDIGITS];
-	char separator_index[MAXCOLDIGITS];
-	char unit[MAXCOLDIGITS];
-	char datefmt[MAXCOLDIGITS];
-	char durationfmt[MAXCOLDIGITS];
-	char cylsizebuf[MAXCOLDIGITS];
-	char startpresbuf[MAXCOLDIGITS];
-	char endpresbuf[MAXCOLDIGITS];
-	char o2buf[MAXCOLDIGITS];
-	char hebuf[MAXCOLDIGITS];
-	char airtempbuf[MAXCOLDIGITS];
-	char watertempbuf[MAXCOLDIGITS];
 	time_t now;
 	struct tm *timep;
 	char curdate[9];
 	char curtime[6];
-	int ret;
+	int ret, i;
 
-	if (numberf >= MAXCOLS || datef >= MAXCOLS || timef >= MAXCOLS || durationf >= MAXCOLS || locationf >= MAXCOLS || gpsf >= MAXCOLS || maxdepthf >= MAXCOLS || meandepthf >= MAXCOLS || buddyf >= MAXCOLS || notesf >= MAXCOLS || weightf >= MAXCOLS || tagsf >= MAXCOLS || cylsizef >= MAXCOLS || startpresf >= MAXCOLS || endpresf >= MAXCOLS || o2f >= MAXCOLS || hef >= MAXCOLS || airtempf >= MAXCOLS || watertempf >= MAXCOLS)
-		return report_error(translate("gettextFromC", "Maximum number of supported columns on CSV import is %d"), MAXCOLS);
 
-	snprintf(numberbuf, MAXCOLDIGITS, "%d", numberf);
-	snprintf(datebuf, MAXCOLDIGITS, "%d", datef);
-	snprintf(timebuf, MAXCOLDIGITS, "%d", timef);
-	snprintf(durationbuf, MAXCOLDIGITS, "%d", durationf);
-	snprintf(locationbuf, MAXCOLDIGITS, "%d", locationf);
-	snprintf(gpsbuf, MAXCOLDIGITS, "%d", gpsf);
-	snprintf(maxdepthbuf, MAXCOLDIGITS, "%d", maxdepthf);
-	snprintf(meandepthbuf, MAXCOLDIGITS, "%d", meandepthf);
-	snprintf(divemasterbuf, MAXCOLDIGITS, "%d", divemasterf);
-	snprintf(buddybuf, MAXCOLDIGITS, "%d", buddyf);
-	snprintf(notesbuf, MAXCOLDIGITS, "%d", notesf);
-	snprintf(weightbuf, MAXCOLDIGITS, "%d", weightf);
-	snprintf(tagsbuf, MAXCOLDIGITS, "%d", tagsf);
-	snprintf(separator_index, MAXCOLDIGITS, "%d", sepidx);
-	snprintf(unit, MAXCOLDIGITS, "%d", units);
-	snprintf(datefmt, MAXCOLDIGITS, "%d", dateformat);
-	snprintf(durationfmt, MAXCOLDIGITS, "%d", durationformat);
-	snprintf(cylsizebuf, MAXCOLDIGITS, "%d", cylsizef);
-	snprintf(startpresbuf, MAXCOLDIGITS, "%d", startpresf);
-	snprintf(endpresbuf, MAXCOLDIGITS, "%d", endpresf);
-	snprintf(o2buf, MAXCOLDIGITS, "%d", o2f);
-	snprintf(hebuf, MAXCOLDIGITS, "%d", hef);
-	snprintf(airtempbuf, MAXCOLDIGITS, "%d", airtempf);
-	snprintf(watertempbuf, MAXCOLDIGITS, "%d", watertempf);
 	time(&now);
 	timep = localtime(&now);
 	strftime(curdate, DATESTR, "%Y%m%d", timep);
@@ -1064,58 +1043,11 @@ int parse_manual_file(const char *filename, int sepidx, int units, int dateforma
 	* is not discarded during the transform, thus prepend time with 1 */
 	strftime(curtime, TIMESTR, "1%H%M", timep);
 
-	params[pnr++] = "numberField";
-	params[pnr++] = numberbuf;
-	params[pnr++] = "dateField";
-	params[pnr++] = datebuf;
-	params[pnr++] = "timeField";
-	params[pnr++] = timebuf;
-	params[pnr++] = "durationField";
-	params[pnr++] = durationbuf;
-	params[pnr++] = "locationField";
-	params[pnr++] = locationbuf;
-	params[pnr++] = "gpsField";
-	params[pnr++] = gpsbuf;
-	params[pnr++] = "maxDepthField";
-	params[pnr++] = maxdepthbuf;
-	params[pnr++] = "meanDepthField";
-	params[pnr++] = meandepthbuf;
-	params[pnr++] = "divemasterField";
-	params[pnr++] = divemasterbuf;
-	params[pnr++] = "buddyField";
-	params[pnr++] = buddybuf;
-	params[pnr++] = "notesField";
-	params[pnr++] = notesbuf;
-	params[pnr++] = "weightField";
-	params[pnr++] = weightbuf;
-	params[pnr++] = "tagsField";
-	params[pnr++] = tagsbuf;
-	params[pnr++] = "date";
-	params[pnr++] = curdate;
-	params[pnr++] = "time";
-	params[pnr++] = curtime;
-	params[pnr++] = "separatorIndex";
-	params[pnr++] = separator_index;
-	params[pnr++] = "units";
-	params[pnr++] = unit;
-	params[pnr++] = "datefmt";
-	params[pnr++] = datefmt;
-	params[pnr++] = "durationfmt";
-	params[pnr++] = durationfmt;
-	params[pnr++] = "cylindersizeField";
-	params[pnr++] = cylsizebuf;
-	params[pnr++] = "startpressureField";
-	params[pnr++] = startpresbuf;
-	params[pnr++] = "endpressureField";
-	params[pnr++] = endpresbuf;
-	params[pnr++] = "o2Field";
-	params[pnr++] = o2buf;
-	params[pnr++] = "heField";
-	params[pnr++] = hebuf;
-	params[pnr++] = "airtempField";
-	params[pnr++] = airtempbuf;
-	params[pnr++] = "watertempField";
-	params[pnr++] = watertempbuf;
+
+	params[pnr++] = strdup("date");
+	params[pnr++] = strdup(curdate);
+	params[pnr++] = strdup("time");
+	params[pnr++] = strdup(curtime);
 	params[pnr++] = NULL;
 
 	if (filename == NULL)
@@ -1126,6 +1058,9 @@ int parse_manual_file(const char *filename, int sepidx, int units, int dateforma
 		return -1;
 
 	ret = parse_xml_buffer(filename, mem.buffer, mem.size, &dive_table, (const char **)params);
+
 	free(mem.buffer);
+	for (i = 0; i < pnr - 2; ++i)
+		free(params[i]);
 	return ret;
 }
