@@ -126,6 +126,39 @@ static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t
 				if (tank.type == DC_TANKVOLUME_IMPERIAL) {
 					dive->cylinder[i].type.size.mliter = rint(tank.volume * 1000);
 					dive->cylinder[i].type.workingpressure.mbar = rint(tank.workpressure * 1000);
+					if (same_string(devdata->model, "Suunto EON Steel")) {
+						/* Suunto EON Steele gets this wrong. Badly.
+						 * but on the plus side it only supports a few imperial sizes,
+						 * so let's try and guess at least the most common ones.
+						 * First, the pressures are off by a constant factor. WTF?
+						 * Then we can round the wet sizes so we get to multiples of 10
+						 * for cuft sizes (as that's all that you can enter) */
+						dive->cylinder[i].type.workingpressure.mbar *= 206.843 / 206.7;
+						char name_buffer[9];
+						int rounded_size = ml_to_cuft(gas_volume(&dive->cylinder[i],
+											 dive->cylinder[i].type.workingpressure));
+						rounded_size = (int)((rounded_size + 5) / 10) * 10;
+						switch (dive->cylinder[i].type.workingpressure.mbar) {
+						case 206843:
+							snprintf(name_buffer, 9, "AL%d", rounded_size);
+							break;
+						case 234422: /* this is wrong - HP tanks tend to be 3440, but Suunto only allows 3400 */
+							snprintf(name_buffer, 9, "HP%d", rounded_size);
+							break;
+						case 179263:
+							snprintf(name_buffer, 9, "LP+%d", rounded_size);
+							break;
+						case 165474:
+							snprintf(name_buffer, 9, "LP%d", rounded_size);
+							break;
+						default:
+							snprintf(name_buffer, 9, "%d cuft", rounded_size);
+							break;
+						}
+						dive->cylinder[i].type.description = copy_string(name_buffer);
+						dive->cylinder[i].type.size.mliter = cuft_to_l(rounded_size) * 1000 /
+										     mbar_to_atm(dive->cylinder[i].type.workingpressure.mbar);
+					}
 				} else if (tank.type == DC_TANKVOLUME_METRIC) {
 					dive->cylinder[i].type.size.mliter = rint(tank.volume * 1000);
 				}
@@ -198,7 +231,6 @@ static void handle_event(struct divecomputer *dc, struct sample *sample, dc_samp
 void
 sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 {
-	unsigned int mm;
 	static unsigned int nsensor = 0;
 	struct divecomputer *dc = userdata;
 	struct sample *sample;
@@ -219,7 +251,10 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 	switch (type) {
 	case DC_SAMPLE_TIME:
 		nsensor = 0;
-		mm = 0;
+
+		// The previous sample gets some sticky values
+		// that may have been around from before, even
+		// if there was no new data
 		if (sample) {
 			sample->in_deco = in_deco;
 			sample->ndl.seconds = ndl;
@@ -227,11 +262,12 @@ sample_cb(dc_sample_type_t type, dc_sample_value_t value, void *userdata)
 			sample->stopdepth.mm = stopdepth;
 			sample->setpoint.mbar = po2;
 			sample->cns = cns;
-			mm = sample->depth.mm;
 		}
+		// Create a new sample.
+		// Mark depth as negative
 		sample = prepare_sample(dc);
 		sample->time.seconds = value.time;
-		sample->depth.mm = mm;
+		sample->depth.mm = -1;
 		finish_sample(dc);
 		break;
 	case DC_SAMPLE_DEPTH:
@@ -935,7 +971,9 @@ const char *do_libdivecomputer_import(device_data_t *data)
 	dc_serial_t *serial_device = NULL;
 
 	if (data->bluetooth_mode) {
+#ifdef BT_SUPPORT
 		rc = dc_serial_qt_open(&serial_device, data->context, data->devname);
+#endif
 #ifdef SERIAL_FTDI
 	} else if (!strcmp(data->devname, "ftdi")) {
 		rc = dc_serial_ftdi_open(&serial_device, data->context);
