@@ -67,6 +67,10 @@ static void dump_pi(struct plot_info *pi)
 int get_maxtime(struct plot_info *pi)
 {
 	int seconds = pi->maxtime;
+
+	int DURATION_THR = (pi->dive_type == FREEDIVING ? 60 : 600);
+	int CEILING = (pi->dive_type == FREEDIVING ? 30 : 60);
+
 	if (prefs.zoomed_plot) {
 		/* Rounded up to one minute, with at least 2.5 minutes to
 		 * spare.
@@ -74,13 +78,13 @@ int get_maxtime(struct plot_info *pi)
 		 * calculate the space dynamically.
 		 * This is seamless since 600/4 = 150.
 		 */
-		if (seconds < 600)
-			return ROUND_UP(seconds + seconds / 4, 60);
+		if (seconds < DURATION_THR)
+			return ROUND_UP(seconds + seconds / 4, CEILING);
 		else
-			return ROUND_UP(seconds + 150, 60);
+			return ROUND_UP(seconds + DURATION_THR/4, CEILING);
 	} else {
 		/* min 30 minutes, rounded up to 5 minutes, with at least 2.5 minutes to spare */
-		return MAX(30 * 60, ROUND_UP(seconds + 150, 60 * 5));
+		return MAX(30 * 60, ROUND_UP(seconds + DURATION_THR/4, CEILING * 5));
 	}
 }
 
@@ -701,31 +705,111 @@ static void populate_cylinder_pressure_data(int idx, int start, int end, struct
 	}
 }
 
+/*
+ * Calculate the sac rate between the two plot entries 'first' and 'last'.
+ *
+ * Everything in between has a cylinder pressure, and it's all the same
+ * cylinder.
+ */
+static int sac_between(struct dive *dive, struct plot_data *first, struct plot_data *last)
+{
+	int airuse;
+	double pressuretime;
+	pressure_t a, b;
+	cylinder_t *cyl;
+	int duration;
+
+	if (first == last)
+		return 0;
+
+	/* Calculate air use - trivial */
+	a.mbar = GET_PRESSURE(first);
+	b.mbar = GET_PRESSURE(last);
+	cyl = dive->cylinder + first->cylinderindex;
+	airuse = gas_volume(cyl, a) - gas_volume(cyl, b);
+	if (airuse <= 0)
+		return 0;
+
+	/* Calculate depthpressure integrated over time */
+	pressuretime = 0.0;
+	do {
+		int depth = (first[0].depth + first[1].depth) / 2;
+		int time = first[1].sec - first[0].sec;
+		double atm = depth_to_atm(depth, dive);
+
+		pressuretime += atm * time;
+	} while (++first < last);
+
+	/* Turn "atmseconds" into "atmminutes" */
+	pressuretime /= 60;
+
+	/* SAC = mliter per minute */
+	return rint(airuse / pressuretime);
+}
+
+/*
+ * Try to do the momentary sac rate for this entry, averaging over one
+ * minute.
+ */
+static void fill_sac(struct dive *dive, struct plot_info *pi, int idx)
+{
+	struct plot_data *entry = pi->entry + idx;
+	struct plot_data *first, *last;
+	int time;
+
+	if (entry->sac)
+		return;
+
+	if (!GET_PRESSURE(entry))
+		return;
+
+	/*
+	 * Try to go back 30 seconds to get 'first'.
+	 * Stop if the sensor changed, or if we went back too far.
+	 */
+	first = entry;
+	time = entry->sec - 30;
+	while (idx > 0) {
+		struct plot_data *prev = first-1;
+		if (prev->cylinderindex != first->cylinderindex)
+			break;
+		if (prev->depth < SURFACE_THRESHOLD && first->depth < SURFACE_THRESHOLD)
+			break;
+		if (prev->sec < time)
+			break;
+		if (!GET_PRESSURE(prev))
+			break;
+		idx--;
+		first = prev;
+	}
+
+	/* Now find an entry a minute after the first one */
+	last = first;
+	time = first->sec + 60;
+	while (++idx < pi->nr) {
+		struct plot_data *next = last+1;
+		if (next->cylinderindex != last->cylinderindex)
+			break;
+		if (next->depth < SURFACE_THRESHOLD && last->depth < SURFACE_THRESHOLD)
+			break;
+		if (next->sec > time)
+			break;
+		if (!GET_PRESSURE(next))
+			break;
+		last = next;
+	}
+
+	/* Ok, now calculate the SAC between 'first' and 'last' */
+	entry->sac = sac_between(dive, first, last);
+}
+
 static void calculate_sac(struct dive *dive, struct plot_info *pi)
 {
 	int i = 0, last = 0;
 	struct plot_data *last_entry = NULL;
 
-	for (i = 0; i < pi->nr; i++) {
-		struct plot_data *entry = pi->entry + i;
-		if (entry->sac)
-			continue;
-		if (!last_entry || last_entry->cylinderindex != entry->cylinderindex) {
-			last = i;
-			last_entry = entry;
-			entry->sac = get_local_sac(entry, pi->entry + i + 1, dive);
-		} else {
-			int j;
-			entry->sac = 0;
-			for (j = last; j < i; j++)
-				entry->sac += get_local_sac(pi->entry + j, pi->entry + j + 1, dive);
-			entry->sac /= (i - last);
-			if (entry->sec - last_entry->sec >= SAC_WINDOW) {
-				last++;
-				last_entry = pi->entry + last;
-			}
-		}
-	}
+	for (i = 0; i < pi->nr; i++)
+		fill_sac(dive, pi, i);
 }
 
 static void populate_secondary_sensor_data(struct divecomputer *dc, struct plot_info *pi)
@@ -1071,7 +1155,9 @@ void create_plot_info_new(struct dive *dive, struct divecomputer *dc, struct plo
 	free((void *)last_pi_entry_new);
 
 	get_dive_gas(dive, &o2, &he, &o2max);
-	if (he > 0) {
+	if (dc->divemode == FREEDIVE){
+		pi->dive_type = FREEDIVE;
+	} else 	if (he > 0) {
 		pi->dive_type = TRIMIX;
 	} else {
 		if (o2)
@@ -1079,6 +1165,7 @@ void create_plot_info_new(struct dive *dive, struct divecomputer *dc, struct plo
 		else
 			pi->dive_type = AIR;
 	}
+
 	last_pi_entry_new = populate_plot_entries(dive, dc, pi);
 
 	check_gas_change_events(dive, dc, pi);   /* Populate the gas index from the gas change events */
@@ -1164,6 +1251,7 @@ static void plot_string(struct plot_info *pi, struct plot_data *entry, struct me
 			put_format(b, translate("gettextFromC", "END: %d%s\nEADD: %d%s\n"), end, depth_unit, eadd, depth_unit);
 			break;
 		case AIR:
+		case FREEDIVING:
 			/* nothing */
 			break;
 		}
