@@ -33,8 +33,6 @@ private:
 	void showExplanation();
 	void exportHtmlInit(const QString &filename);
 	void exportHTMLsettings(const QString &filename);
-	void exportHTMLstatistics(const QString &filename);
-	void copy_and_overwrite(const QString &fileName, const QString &newName);
 	void export_depths(const char *filename, const bool selected_only);
 };
 
