@@ -11,6 +11,10 @@
 #include "configuredivecomputerdialog.h"
 #include "ui_downloadfromdivecomputer.h"
 
+#if defined(BT_SUPPORT)
+#include "btdeviceselectiondialog.h"
+#endif
+
 class QStringListModel;
 
 class DownloadThread : public QThread {
@@ -79,7 +83,11 @@ slots:
 	void checkDumpFile(int state);
 	void pickDumpFile();
 	void pickLogFile();
-
+#if defined(BT_SUPPORT)
+	void enableBluetoothMode(int state);
+	void selectRemoteBluetoothDevice();
+	void bluetoothSelectionDialogIsFinished(int result);
+#endif
 private:
 	void markChildrenAsDisabled();
 	void markChildrenAsEnabled();
@@ -104,6 +112,9 @@ private:
 	bool dumpWarningShown;
 	OstcFirmwareCheck *ostcFirmwareCheck;
 	DiveImportedModel *diveImportedModel;
+#if defined(BT_SUPPORT)
+	BtDeviceSelectionDialog *btDeviceSelectionDialog;
+#endif
 
 public:
 	bool preferDownloaded();
