@@ -10,12 +10,20 @@
 #include "ruleritem.h"
 #include "tankitem.h"
 #include "pref.h"
+#include "divepicturewidget.h"
+#include "diveplannermodel.h"
+#include "models.h"
+#include "divepicturemodel.h"
+#include "maintab.h"
+#include "diveplanner.h"
+
 #include <libdivecomputer/parser.h>
 #include <QScrollBar>
 #include <QtCore/qmath.h>
 #include <QMessageBox>
 #include <QInputDialog>
 #include <QDebug>
+#include <QWheelEvent>
 
 #ifndef QT_NO_DEBUG
 #include <QTableView>
@@ -67,7 +75,7 @@ ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 	background(new DivePixmapItem()),
 	backgroundFile(":poster"),
 	toolTipItem(new ToolTipItem()),
-	isPlotZoomed(prefs.zoomed_plot),
+	isPlotZoomed(prefs.zoomed_plot),// no! bad use of prefs. 'PreferencesDialog::loadSettings' NOT CALLED yet.
 	profileYAxis(new DepthAxis()),
 	gasYAxis(new PartialGasPressureAxis()),
 	temperatureAxis(new TemperatureAxis()),
@@ -79,6 +87,7 @@ ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 	gasPressureItem(new DiveGasPressureItem()),
 	diveComputerText(new DiveTextItem()),
 	diveCeiling(new DiveCalculatedCeiling()),
+	gradientFactor(new DiveTextItem()),
 	reportedCeiling(new DiveReportedCeiling()),
 	pn2GasItem(new PartialPressureGasItem()),
 	pheGasItem(new PartialPressureGasItem()),
@@ -102,6 +111,9 @@ ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 	shouldCalculateMaxDepth(true),
 	fontPrintScale(1.0)
 {
+	// would like to be able to ASSERT here that PreferencesDialog::loadSettings has been called.
+	isPlotZoomed = prefs.zoomed_plot; // now it seems that 'prefs' has loaded our preferences
+
 	memset(&plotInfo, 0, sizeof(plotInfo));
 
 	setupSceneAndFlags();
@@ -192,6 +204,7 @@ void ProfileWidget2::addItemsToScene()
 	diveComputerText->setData(SUBSURFACE_OBJ_DATA, SUBSURFACE_OBJ_DC_TEXT);
 	scene()->addItem(diveComputerText);
 	scene()->addItem(diveCeiling);
+	scene()->addItem(gradientFactor);
 	scene()->addItem(reportedCeiling);
 	scene()->addItem(pn2GasItem);
 	scene()->addItem(pheGasItem);
@@ -275,6 +288,12 @@ void ProfileWidget2::setupItemOnScene()
 	rulerItem->setAxis(timeAxis, profileYAxis);
 	tankItem->setHorizontalAxis(timeAxis);
 
+	// show the gradient factor at the top in the center
+	gradientFactor->setY(0);
+	gradientFactor->setX(50);
+	gradientFactor->setBrush(getColor(PRESSURE_TEXT));
+	gradientFactor->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
+
 	setupItem(reportedCeiling, timeAxis, profileYAxis, dataModel, DivePlotDataModel::CEILING, DivePlotDataModel::TIME, 1);
 	setupItem(diveCeiling, timeAxis, profileYAxis, dataModel, DivePlotDataModel::CEILING, DivePlotDataModel::TIME, 1);
 	for (int i = 0; i < 16; i++) {
@@ -326,12 +345,12 @@ void ProfileWidget2::setupItemOnScene()
 	replotEnabled = true;
 }
 
-void ProfileWidget2::replot()
+void ProfileWidget2::replot(struct dive *d)
 {
 	if (!replotEnabled)
 		return;
 	dataModel->clear();
-	plotDive(0, true); // simply plot the displayed_dive again
+	plotDive(d, true);
 }
 
 void ProfileWidget2::setupItemSizes()
@@ -458,6 +477,15 @@ void ProfileWidget2::setupSceneAndFlags()
 	background->setFlag(QGraphicsItem::ItemIgnoresTransformations);
 }
 
+void ProfileWidget2::resetZoom()
+{
+	if (!zoomLevel)
+		return;
+	const qreal defScale = 1.0 / qPow(zoomFactor, (qreal)zoomLevel);
+	scale(defScale, defScale);
+	zoomLevel = 0;
+}
+
 // Currently just one dive, but the plan is to enable All of the selected dives.
 void ProfileWidget2::plotDive(struct dive *d, bool force)
 {
@@ -481,13 +509,16 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 
 		// this copies the dive and makes copies of all the relevant additional data
 		copy_dive(d, &displayed_dive);
+		gradientFactor->setText(QString("GF %1/%2").arg(prefs.gflow).arg(prefs.gfhigh));
 	} else {
 		DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
 		plannerModel->createTemporaryPlan();
-		if (!plannerModel->getDiveplan().dp) {
+		struct diveplan &diveplan = plannerModel->getDiveplan();
+		if (!diveplan.dp) {
 			plannerModel->deleteTemporaryPlan();
 			return;
 		}
+		gradientFactor->setText(QString("GF %1/%2").arg(diveplan.gflow).arg(diveplan.gfhigh));
 	}
 
 	// special handling for the first time we display things
@@ -499,11 +530,7 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 	}
 
 	// restore default zoom level
-	if (zoomLevel) {
-		const qreal defScale = 1.0 / qPow(zoomFactor, (qreal)zoomLevel);
-		scale(defScale, defScale);
-		zoomLevel = 0;
-	}
+	resetZoom();
 
 	// reset some item visibility on printMode changes
 	toolTipItem->setVisible(!printMode);
@@ -608,6 +635,17 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 	eventItems.clear();
 	struct event *event = currentdc->events;
 	while (event) {
+		// if print mode is selected only draw headings, SP change, gas events or bookmark event
+		if (printMode) {
+			if (same_string(event->name, "") ||
+			    !(strcmp(event->name, "heading") == 0 ||
+			      (same_string(event->name, "SP change") && event->time.seconds == 0) ||
+			      event_is_gaschange(event) ||
+			      event->type == SAMPLE_EVENT_BOOKMARK)) {
+				event = event->next;
+				continue;
+			}
+		}
 		DiveEventItem *item = new DiveEventItem();
 		item->setHorizontalAxis(timeAxis);
 		item->setVerticalAxis(profileYAxis);
@@ -644,8 +682,10 @@ void ProfileWidget2::plotDive(struct dive *d, bool force)
 	// so if we are calculation TTS / NDL then let's force that off.
 	if (measureDuration.elapsed() > 1000 && prefs.calcndltts) {
 		MainWindow::instance()->turnOffNdlTts();
-		MainWindow::instance()->showError(tr("Show NDL / TTS was disabled because of excessive processing time"));
+		MainWindow::instance()->getNotificationWidget()->showNotification(tr("Show NDL / TTS was disabled because of excessive processing time"), KMessageWidget::Error);
 	}
+	MainWindow::instance()->getNotificationWidget()->showNotification(get_error_string(), KMessageWidget::Error);
+
 }
 
 void ProfileWidget2::recalcCeiling()
@@ -727,6 +767,8 @@ void ProfileWidget2::resizeEvent(QResizeEvent *event)
 
 void ProfileWidget2::mousePressEvent(QMouseEvent *event)
 {
+	if (zoomLevel)
+		return;
 	QGraphicsView::mousePressEvent(event);
 	if (currentState == PLAN)
 		shouldCalculateMaxTime = false;
@@ -734,18 +776,24 @@ void ProfileWidget2::mousePressEvent(QMouseEvent *event)
 
 void ProfileWidget2::divePlannerHandlerClicked()
 {
+	if (zoomLevel)
+		return;
 	shouldCalculateMaxDepth = false;
 	replot();
 }
 
 void ProfileWidget2::divePlannerHandlerReleased()
 {
+	if (zoomLevel)
+		return;
 	shouldCalculateMaxDepth = true;
 	replot();
 }
 
 void ProfileWidget2::mouseReleaseEvent(QMouseEvent *event)
 {
+	if (zoomLevel)
+		return;
 	QGraphicsView::mouseReleaseEvent(event);
 	if (currentState == PLAN) {
 		shouldCalculateMaxTime = true;
@@ -878,6 +926,7 @@ void ProfileWidget2::setEmptyState()
 	toolTipItem->setVisible(false);
 	diveComputerText->setVisible(false);
 	diveCeiling->setVisible(false);
+	gradientFactor->setVisible(false);
 	reportedCeiling->setVisible(false);
 	rulerItem->setVisible(false);
 	tankItem->setVisible(false);
@@ -1004,6 +1053,7 @@ void ProfileWidget2::setProfileState()
 	diveComputerText->setPos(itemPos.dcLabel.on);
 
 	diveCeiling->setVisible(prefs.calcceiling);
+	gradientFactor->setVisible(prefs.calcceiling);
 	reportedCeiling->setVisible(prefs.dcceiling);
 
 	if (prefs.calcalltissues) {
@@ -1039,6 +1089,7 @@ void ProfileWidget2::clearHandlers()
 	if (handles.count()) {
 		foreach (DiveHandler *handle, handles) {
 			scene()->removeItem(handle);
+			delete handle;
 		}
 		handles.clear();
 	}
@@ -1054,6 +1105,7 @@ void ProfileWidget2::setAddState()
 	if (currentState == ADD)
 		return;
 
+	clearHandlers();
 	setProfileState();
 	mouseFollowerHorizontal->setVisible(true);
 	mouseFollowerVertical->setVisible(true);
@@ -1079,6 +1131,7 @@ void ProfileWidget2::setAddState()
 	/* show the same stuff that the profile shows. */
 	currentState = ADD; /* enable the add state. */
 	diveCeiling->setVisible(true);
+	gradientFactor->setVisible(true);
 	setBackgroundBrush(QColor("#A7DCFF"));
 }
 
@@ -1112,6 +1165,7 @@ void ProfileWidget2::setPlanState()
 	/* show the same stuff that the profile shows. */
 	currentState = PLAN; /* enable the add state. */
 	diveCeiling->setVisible(true);
+	gradientFactor->setVisible(true);
 	setBackgroundBrush(QColor("#D7E3EF"));
 }
 
@@ -1133,7 +1187,7 @@ struct plot_data *ProfileWidget2::getEntryFromPos(QPointF pos)
 {
 	// find the time stamp corresponding to the mouse position
 	int seconds = timeAxis->valueAt(pos);
-	struct plot_data *entry;
+	struct plot_data *entry = NULL;
 
 	for (int i = 0; i < plotInfo.nr; i++) {
 		entry = plotInfo.entry + i;
@@ -1206,6 +1260,10 @@ void ProfileWidget2::contextMenuEvent(QContextMenuEvent *event)
 	setpointAction->setData(event->globalPos());
 	QAction *action = m.addAction(tr("Add bookmark"), this, SLOT(addBookmark()));
 	action->setData(event->globalPos());
+
+	if (same_string(current_dc->model, "manually added dive"))
+		QAction *editProfileAction = m.addAction(tr("Edit the profile"), MainWindow::instance(), SLOT(editCurrentDive()));
+
 	if (DiveEventItem *item = dynamic_cast<DiveEventItem *>(sceneItem)) {
 		action = new QAction(&m);
 		action->setText(tr("Remove event"));
@@ -1307,7 +1365,7 @@ void ProfileWidget2::hideEvents()
 	if (QMessageBox::question(MainWindow::instance(),
 				  TITLE_OR_TEXT(tr("Hide events"), tr("Hide all %1 events?").arg(event->name)),
 				  QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok) {
-		if (event->name) {
+		if (!same_string(event->name, "")) {
 			for (int i = 0; i < evn_used; i++) {
 				if (same_string(event->name, ev_namelist[i].ev_name)) {
 					ev_namelist[i].plot_ev = false;
@@ -1375,8 +1433,21 @@ void ProfileWidget2::changeGas()
 
 	// backup the things on the dataModel, since we will clear that out.
 	struct gasmix gasmix;
-	int seconds = timeAxis->valueAt(scenePos);
+	qreal sec_val = timeAxis->valueAt(scenePos);
 
+	// no gas changes before the dive starts
+	unsigned int seconds = (sec_val < 0.0) ? 0 : (unsigned int)sec_val;
+
+	// if there is a gas change at this time stamp, remove it before adding the new one
+	struct event *gasChangeEvent = current_dc->events;
+	while ((gasChangeEvent = get_next_event(gasChangeEvent, "gaschange")) != NULL) {
+		if (gasChangeEvent->time.seconds == seconds) {
+			remove_event(gasChangeEvent);
+			gasChangeEvent = current_dc->events;
+		} else {
+			gasChangeEvent = gasChangeEvent->next;
+		}
+	}
 	validate_gas(gas.toUtf8().constData(), &gasmix);
 	QRegExp rx("\\(\\D*(\\d+)");
 	int tank;
@@ -1407,6 +1478,17 @@ bool ProfileWidget2::getPrintMode()
 void ProfileWidget2::setPrintMode(bool mode, bool grayscale)
 {
 	printMode = mode;
+	resetZoom();
+
+	// set printMode for axes
+	profileYAxis->setPrintMode(mode);
+	gasYAxis->setPrintMode(mode);
+	temperatureAxis->setPrintMode(mode);
+	timeAxis->setPrintMode(mode);
+	cylinderPressureAxis->setPrintMode(mode);
+	heartBeatAxis->setPrintMode(mode);
+	percentageAxis->setPrintMode(mode);
+
 	isGrayscale = mode ? grayscale : false;
 	mouseFollowerHorizontal->setVisible(!mode);
 	mouseFollowerVertical->setVisible(!mode);
@@ -1415,6 +1497,7 @@ void ProfileWidget2::setPrintMode(bool mode, bool grayscale)
 void ProfileWidget2::setFontPrintScale(double scale)
 {
 	fontPrintScale = scale;
+	emit fontPrintScaleChanged(scale);
 }
 
 double ProfileWidget2::getFontPrintScale()
@@ -1504,6 +1587,7 @@ void ProfileWidget2::repositionDiveHandlers()
 {
 	DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
 	// Re-position the user generated dive handlers
+	struct gasmix mix, lastmix;
 	for (int i = 0; i < plannerModel->rowCount(); i++) {
 		struct divedatapoint datapoint = plannerModel->at(i);
 		if (datapoint.time == 0) // those are the magic entries for tanks
@@ -1528,8 +1612,9 @@ void ProfileWidget2::repositionDiveHandlers()
 		QLineF line(p1, p2);
 		QPointF pos = line.pointAt(0.5);
 		gases[i]->setPos(pos);
-		gases[i]->setVisible(datapoint.entered);
-		gases[i]->setText(dpGasToStr(plannerModel->at(i)));
+		gases[i]->setText(get_divepoint_gas_string(datapoint));
+		gases[i]->setVisible(datapoint.entered &&
+				(i == 0 || gases[i]->text() != gases[i-1]->text()));
 	}
 }
 
@@ -1655,7 +1740,7 @@ void ProfileWidget2::keyRightAction()
 		if (DiveHandler *handler = qgraphicsitem_cast<DiveHandler *>(i)) {
 			int row = handles.indexOf(handler);
 			divedatapoint dp = plannerModel->at(row);
-			if (dp.time / 60 >= timeAxis->maximum())
+			if (dp.time / 60.0 >= timeAxis->maximum())
 				continue;
 
 			// don't overlap positions.
@@ -1740,6 +1825,8 @@ void ProfileWidget2::plotPictures()
 			y = 10;
 		else if (fabs(x - lastX) < 4)
 			y = lastY + 3;
+		else
+			y = 10;
 		lastX = x;
 		lastY = y;
 		item->setPos(x, y);
