@@ -40,14 +40,17 @@ extern stats_t *stats_yearly;
 extern stats_t *stats_monthly;
 extern stats_t *stats_by_trip;
 
-extern char *get_time_string(int seconds, int maxdays);
+extern char *get_time_string_s(int seconds, int maxdays, bool freediving);
 extern char *get_minutes(int seconds);
 extern void process_all_dives(struct dive *dive, struct dive **prev_dive);
 extern void get_selected_dives_text(char *buffer, int size);
 extern void get_gas_used(struct dive *dive, volume_t gases[MAX_CYLINDERS]);
-extern char *get_gaslist(struct dive *dive);
 extern void process_selected_dives(void);
+void selected_dives_gas_parts(volume_t *o2_tot, volume_t *he_tot);
 
+inline char *get_time_string(int seconds, int maxdays) {
+	return get_time_string_s( seconds,  maxdays, false);
+}
 #ifdef __cplusplus
 }
 #endif
