@@ -16,6 +16,8 @@ extern bool imported;
 void setup_system_prefs(void);
 void parse_argument(const char *arg);
 void free_prefs(void);
+void copy_prefs(struct preferences *src, struct preferences *dest);
+void print_files(void);
 
 #ifdef __cplusplus
 }
